# -*- coding: utf-8 -*-
# This file is part of cernopendata-client.
#
# Copyright (C) 2019, 2020, 2022, 2024 CERN.
#
# cernopendata-client is free software; you can redistribute it and/or modify
# it under the terms of the GPLv3 license; see LICENSE file for more details.

"""Version information for cernopendata-client.

This file is imported by ``cernopendata-client.__init__`` and parsed by
``setup.py``.
"""

from __future__ import absolute_import, print_function

__version__ = "0.4.0"
